/***************************************************************************
 *   Copyright (C) 2005 by David Lopez Vinacua                             *
 *   david@bajavista.com                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

/* * * * * * * * * * * * * * * * * * * * * * * *
 *  ip2co.c                                    *
 *  Tools to work with ip-to-country.csv data  *
 * * * * * * * * * * * * * * * * * * * * * * * *
 *
 * Created:
 *  120405 David Lopez
 * Modified:
 *
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "ip2co.h"

t_ip2coRB *ipDB = NULL;

short FillIP2CO(char *s, t_ip2coR *r)
/* Parses s to get r's fields. Returns 0 if Ok, else error */
{
  char         *c;
  u32           ip;
  
  if (!s || !r)
     return -1;
  bzero(r,sizeof(t_ip2coR));
  c=s;
  while (*c && (*c!='"'))
    c++;
  if (! *c)
     return -2;
  // Let's get the base IP
  c++;          // Jump over the quotes
  ip=0;
  while (*c && (*c>='0') && (*c<='9'))
    {
    ip=10*ip + (*c-'0');
    c++;
    }
  r->ip1=ip;
  // Let's get the limit IP
  while (*c && (*c!=','))
    c++;
  while (*c && (*c!='"'))
    c++;
  if (! *c)
     return -2;
  c++;          // Jump over the quotes
  ip=0;
  while (*c && (*c>='0') && (*c<='9'))
    {
    ip=10*ip + (*c-'0');
    c++;
    }
  r->ip2=ip;
  // Now the short country code (what we are looking for)
  while (*c && (*c!=','))
    c++;
  while (*c && (*c!='"'))
    c++;
  if (! *c)
     return -2;
  c++;          // Jump over the quotes
  if (*(c+1))
     {
     r->cc[0]=*c;
     r->cc[1]=*(c+1);
     return 0;
     }
  return -3;
}

short ReadDB(char *FileName)
/* Reads given file, that should be a CSV, and creates a DB with values */
{
  t_ip2coRB    *cs,*ps;
  FILE         *f;
  char          s[256];
  t_ip2coR      cr;
  
  if (ipDB)         // Already assigned
     return -2;
  f=fopen(FileName,"r");
  if (!f)           // No file to open...
     return -3;
  ps=NULL;
  cs=(t_ip2coRB *) malloc(sizeof(t_ip2coRB));
  if (!cs)          // Alloc failure
     return -32;
  cs->Next=NULL;
  cs->NumR=0;
  ipDB=cs;
  while (fgets(&s[0],sizeof(s),f))
    {
    if (FillIP2CO(&s[0],&cr))
       break;
    cs->d[cs->NumR]=cr;
    cs->NumR++;
    if (cs->NumR>=k_ip2coMaxR)
       {
       ps=cs;
       cs=(t_ip2coRB *) malloc(sizeof(t_ip2coRB));
       if (!cs)     // Alloc failure
          return -32;
       cs->Next=NULL;
       cs->NumR=0;
       ps->Next=cs;
       }
    }
  fclose(f);
  return 0;
}
