/***************************************************************************
 *   Copyright (C) 2005 by David Lopez Vinacua                             *
 *   david@bajavista.com                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

/* * * * * * * * * * * * * * * * * * * * * * * *
 *  main.c                                     *
 *  Tools to work with ip-to-country.csv data  *
 * * * * * * * * * * * * * * * * * * * * * * * *
 *
 * Created:
 *  120405 David Lopez
 * Modified:
 *
 */

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <errno.h>
#include <string.h>

#include "ip2co.h"

#define DataPath "/usr/local/ip2co"
#define ip_FileName "ip-to-country"

void Help(void)
{
  printf("Usage:\n");
  printf(" ip2co <dotted_quad_notation_IP>\n");
  exit(1);
}

short LoadDB(char *Path, char *FileName)
/* Checks if text file is newer than binary one (then regenerates the former). And loads DB */
{
  char          fn1[256],fn2[256];
  struct stat   st1,st2;
  FILE         *f;
  t_ip2coRB    *r,*ps;
  int           rb;
  
  sprintf(fn1,"%s/%s.csv",Path,FileName);
  if (stat(fn1,&st1))
     {
     fprintf(stderr,"Can't stat %s (%s)\n",fn1,strerror(errno));
     return -2;
     }
  sprintf(fn2,"%s/%s.dat",Path,FileName);
  if (stat(fn2,&st2) || (st1.st_mtime>st2.st_mtime))
     {          // Generate a binary version (and save it for future uses)
     if (ReadDB(&fn1[0]))
        {
        fprintf(stderr,"Failed to read %s (%s)\n",fn1,strerror(errno));
        return -5;
        }
     fprintf(stdout,"Generating a binary version of IP database...\n");
     f=fopen(fn2,"w");
     if (f)
        {
        r=ipDB;
        while (r)
          {
          if (fwrite(((char *)r)+sizeof(void *),sizeof(t_ip2coRB)-sizeof(void *),1,f)!=1)
             {
             fprintf(stderr,"Warning: Failed while writing to %s (%s)\n",fn2,strerror(errno));
             break;
             }
          r=r->Next;
          }
        fclose(f);
        }
     else
        fprintf(stderr,"Warning: Failed to write to %s (%s)\n",fn2,strerror(errno));
     }
  else          // Load the binary version
     {
     f=fopen(fn2,"r");
     if (f)
        {
        fprintf(stdout,"Loading a previous binary version of IP database...\n");
        r=(t_ip2coRB *)malloc(sizeof(t_ip2coRB));
        if (!r)
           {
           fclose(f);
           fprintf(stderr,"Failed to alloc while reading %s (%s)\n",fn2,strerror(errno));
           return -32;
           }
        r->Next=NULL;
        ipDB=r;
        while ( (rb=fread(((char *)r)+sizeof(void *),1,sizeof(t_ip2coRB)-sizeof(void *),f)) )
          {
          if (rb==sizeof(t_ip2coRB)-sizeof(void *))
             {
             ps=r;
             r=(t_ip2coRB *)malloc(sizeof(t_ip2coRB));
             if (!r)
                {
                fclose(f);
                fprintf(stderr,"Failed to alloc while reading %s (%s)\n",fn2,strerror(errno));
                return -32;
                }
             r->Next=NULL;
             ps->Next=r;
             }
          }
        fclose(f);
        }
     else
        {
        fprintf(stderr,"Failed to open %s (%s)\n",fn2,strerror(errno));
        return -6;
        }
     }
  return 0;
}

void Search(u32 ip)
/* Looks for that IP into the database, and shows the country code it belongs */
{
  t_ip2coRB    *r;
  short         cp;

  if (!ipDB)
     return;
  r=ipDB;
  while (r)
    {
    cp=0;
    while (cp<r->NumR)
      {
      if ( (r->d[cp].ip1<=ip) && (r->d[cp].ip2>=ip) )
         {
         fprintf(stdout,"%u.%u.%u.%u is located in %c%c\n",ip>>24,(ip>>16) & 0xff,(ip>>8) & 0xff,ip & 0xff,r->d[cp].cc[0],r->d[cp].cc[1]);
         return;
         }
      cp++;
      }
    r=r->Next;
    }
  fprintf(stdout,"Not found.\n");
}

int main(int argc, char *argv[])
{
  u32       ip;
  unsigned short v;
  char     *c;
  
  if (argc<2)
     Help();
  ip=0;
  c=argv[1];
  v=0;
  while (*c && (*c>='0') && (*c<='9'))
    {
    v=10*v + (*c-'0');
    c++;
    }
  ip=65536*256*v;
  if (*c!='.')
     Help();
  c++;          // Jump over the dot
  v=0;
  while (*c && (*c>='0') && (*c<='9'))
    {
    v=10*v + (*c-'0');
    c++;
    }
  ip+=65536*v;
  if (*c!='.')
     Help();
  c++;          // Jump over the dot
  v=0;
  while (*c && (*c>='0') && (*c<='9'))
    {
    v=10*v + (*c-'0');
    c++;
    }
  ip+=256*v;
  if (*c!='.')
     Help();
  c++;          // Jump over the dot
  v=0;
  while (*c && (*c>='0') && (*c<='9'))
    {
    v=10*v + (*c-'0');
    c++;
    }
  ip+=v;
  if (LoadDB(DataPath,ip_FileName)==0)
     Search(ip);
  return 0;
}
